(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue'), require('@devops-web/utils'), require('lodash-es')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue', '@devops-web/utils', 'lodash-es'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.DevopsHooks = {}, global.Vue, global.DevopsUtils, global._));
})(this, (function (exports, vue, utils, lodashEs) { 'use strict';

  function useMemo(getValue, condition, shouldUpdate) {
      const cacheRef = vue.ref(getValue());
      vue.watch(condition, (next, pre) => {
          if (shouldUpdate) {
              if (shouldUpdate(next, pre)) {
                  cacheRef.value = getValue();
              }
          }
          else {
              cacheRef.value = getValue();
          }
      });
      return cacheRef;
  }

  /**
   * a-select 组件 options 数据的 hooks 工具
   *
   * @param fun options 数据或者一个返回 options 数据的接口函数
   * @param _config 配置
   * @param _config.valueKey 当 option 的 value 为 object 类型时，需要指定唯一标识的字段名，默认：`value`
   * @param _config.immediate 是否立即调用一次 loadOptions，默认：`false`
   * @param _config.fallback 当 value 无匹配 option 时，option 的回退函数
   *
   * @example
   * ```typescript
   * const { options } = useSelectOptions([{ label:'label-1', value:'1' }])
   *
   * // 传入函数
   * const { options, getOptions, getLabels } = useSelectOptions((params) => {
   *    console.log('传入的参数：',params)
   *    return Promise.resolve([{ label: 'label-1', value: 1 }])
   * })
   * loadOptions({type:1}) // 调用传入的函数
   *
   * // 传入函数，并立即调用loadOptions()
   * const { options, getOptions, getLabels } = useSelectOptions(() => {
   *    return Promise.resolve([{ label: 'label-1', value: 1 }])
   * },{immediate:true})
   *
   * // value为object类型
   * const { options, getOptions, getLabels } = useSelectOptions([{ label: 'label-1', value: {value:1} }])
   * getLabels(1) // label-1
   * getLabels({value:1}) // label-1
   *
   *
   * // 配置fallback，getLabels，getOptions数据来源于fallback
   * const { options, getOptions, getLabels } = useSelectOptions([], {
   *      fallback(value) {
   *        if (value === 1) {
   *          return { value, label: 'label-1' }
   *        } else {
   *          return { value, label: undefined }
   *        }
   *       }
   *    })
   * getLabels(1) // label-1
   * getOptions(1) // { value:1, label: 'label-1' }
   * getLabels(2) // undefined
   * getOptions(2) // { value:2, label: undefined }
   *
   * ```
   */
  function useSelectOptions(fun, _config) {
      const defaultConfig = { valueKey: 'value', immediate: false };
      const config = Object.assign(Object.assign({}, defaultConfig), _config);
      const loading = vue.ref(false);
      const cacheOptionsMap = new Map();
      const options = vue.ref([]);
      function loadOptions(...params) {
          loading.value = true;
          if (utils.isFunction(fun)) {
              return fun(...params)
                  .then((data) => {
                  options.value = data;
                  pushCacheOptions(data);
                  return data;
              })
                  .catch((err) => {
                  options.value = [];
                  return Promise.reject(err);
              })
                  .finally(() => {
                  loading.value = false;
              });
          }
          return Promise.resolve([]);
      }
      if (utils.isArray(fun)) {
          options.value = fun;
          pushCacheOptions(fun);
      }
      if (config.immediate) {
          loadOptions();
      }
      function isSelectOptionGroup(option) {
          return option.isGroup === true;
      }
      function pushCacheOptions(data) {
          const dataCopy = [...data];
          while (dataCopy.length > 0) {
              const option = dataCopy.pop();
              if (utils.isDef(option)) {
                  if (utils.isObject(option)) {
                      if (isSelectOptionGroup(option)) {
                          dataCopy.push(...option.options);
                      }
                      else {
                          const { value } = option;
                          if (utils.isObject(value)) {
                              cacheOptionsMap.set(value[config.valueKey], option);
                          }
                          else if (utils.isDef(value)) {
                              cacheOptionsMap.set(value, option);
                          }
                          else {
                              console.error('option：不存在value字段', option);
                          }
                      }
                  }
                  else {
                      cacheOptionsMap.set(option, { label: String(option), value: option });
                  }
              }
          }
      }
      function setOptions(data) {
          options.value = data;
      }
      function getOptionFromCacheOrFallback(value) {
          let option;
          if (utils.isObject(value)) {
              option = cacheOptionsMap.get(value[config.valueKey]);
          }
          else {
              option = cacheOptionsMap.get(value);
          }
          if (utils.isUnDef(option) && utils.isFunction(config.fallback)) {
              option = config.fallback(value);
          }
          return option;
      }
      function getOptions(value) {
          if (utils.isUnDef(value)) {
              return undefined;
          }
          if (Array.isArray(value)) {
              return value.map(getOptionFromCacheOrFallback);
          }
          return getOptionFromCacheOrFallback(value);
      }
      function getLabels(value) {
          if (utils.isUnDef(value)) {
              return undefined;
          }
          const options = getOptions(value);
          if (utils.isArray(options)) {
              return options.map((el) => {
                  if (utils.isDef(el) && utils.isObject(el)) {
                      return el.label;
                  }
                  return el;
              });
          }
          if (utils.isUnDef(options)) {
              return undefined;
          }
          if (utils.isObject(options)) {
              return options.label;
          }
          return options;
      }
      function fallbackOption(value) {
          const option = getOptions(value);
          if (utils.isDef(option)) {
              return option;
          }
          if (config.fallback) {
              return config.fallback(value);
          }
          return {
              value,
              label: undefined
          };
      }
      return {
          loading,
          options,
          /**
           * 加载options，传入immediate=true，会在初始化的时候调用一次
           */
          loadOptions,
          setOptions,
          /**
           * 根据value获取对应的option，option会从之前缓存的options选项中获取，缓存的选项没有会调用传入的fallback函数并返回fallback返回的回退option，如果还没有返回undefined
           */
          getOptions,
          /**
           * 根据value获取对应的label，label会从之前缓存的options选项中获取，缓存的选项没有会调用传入的fallback函数并返回fallback返回的回退option.label，如果还没有返回undefined
           */
          getLabels,
          fallbackOption
      };
  }

  /**
   * 过滤columns，去掉visible===false的列，和children为空的列
   */
  function filterColumns(columns) {
      const filteredNodes = [];
      for (const column of columns) {
          if (column.visible !== false) {
              const { children } = column;
              if (Array.isArray(children) && children.length > 0) {
                  const filteredChildren = filterColumns(children);
                  if (filteredChildren.length > 0) {
                      filteredNodes.push(Object.assign(Object.assign({}, column), { children: filteredChildren }));
                  }
              }
              else {
                  filteredNodes.push(Object.assign({}, column));
              }
          }
      }
      return filteredNodes;
  }
  /**
   * arco-design 扩展表格columns，增加visible，minWidth两个配置项
   * @param columns
   * @returns
   */
  function useColumns(columns) {
      const originColumns = lodashEs.cloneDeep(columns);
      const originColumnsRef = vue.ref(columns);
      const tableInstance = vue.ref(null);
      let lastWidth = 0;
      const observer = new ResizeObserver((entries) => {
          const entry = entries[0];
          if (entry.contentRect.width !== lastWidth) {
              lastWidth = entry.contentRect.width;
              updateColumnsWidth();
          }
      });
      vue.watch(tableInstance, () => {
          observer.disconnect();
          if (tableInstance.value) {
              observer.observe(tableInstance.value.$el);
          }
      });
      const columnsRef = vue.ref([]);
      vue.watchEffect(() => {
          columnsRef.value = filterColumns(originColumnsRef.value);
          vue.nextTick(() => {
              updateColumnsWidth();
          });
      });
      const doFlattenColumns = (columns) => {
          const result = [];
          columns.forEach((column) => {
              if (column.children && column.children.length > 0) {
                  result.push(...doFlattenColumns(column.children));
              }
              else {
                  result.push(column);
              }
          });
          return result;
      };
      function getLeafColumns() {
          return doFlattenColumns(columnsRef.value);
      }
      function updateColumnsWidth() {
          if (!tableInstance.value) {
              return;
          }
          const borderWidth = tableInstance.value.bordered ? 2 : 0;
          const bodyWidth = tableInstance.value.$el.clientWidth - borderWidth;
          let bodyMinWidth = 0;
          const flattenColumns = getLeafColumns();
          const flexColumns = flattenColumns.filter((column) => typeof column.width !== 'number');
          if (flexColumns.length > 0) {
              flattenColumns.forEach((column) => {
                  bodyMinWidth += Number(column.width || column.minWidth || 80);
              });
              if (bodyMinWidth <= bodyWidth) {
                  const totalFlexWidth = bodyWidth - bodyMinWidth;
                  if (flexColumns.length === 1) {
                      flexColumns[0].width = Number(flexColumns[0].minWidth || 80) + totalFlexWidth;
                  }
                  else {
                      const allColumnsWidth = flexColumns.reduce((prev, column) => prev + Number(column.minWidth || 80), 0);
                      const flexWidthPerPixel = totalFlexWidth / allColumnsWidth;
                      let noneFirstWidth = 0;
                      flexColumns.forEach((column, index) => {
                          if (index === 0)
                              return;
                          const flexWidth = Math.floor(Number(column.minWidth || 80) * flexWidthPerPixel);
                          noneFirstWidth += flexWidth;
                          column.width = Number(column.minWidth || 80) + flexWidth;
                      });
                      flexColumns[0].width = Number(flexColumns[0].minWidth || 80) + totalFlexWidth - noneFirstWidth;
                  }
              }
              else {
                  flexColumns.forEach((column) => {
                      column.width = Number(column.minWidth);
                  });
              }
          }
          else {
              flattenColumns.forEach((column) => {
                  if (!column.width && !column.minWidth) {
                      column.width = 80;
                  }
                  else {
                      column.width = Number(column.width || column.minWidth);
                  }
              });
          }
      }
      function resetColumns() {
          originColumnsRef.value = originColumns;
      }
      function replaceColumnByPath(path, column) {
          utils.setValueByPath(originColumnsRef.value, path, column);
      }
      function replaceColumnByDataIndex(dataIndex, column) {
          utils.traverseTreeBFS(originColumnsRef.value, (node) => {
              if (node.dataIndex === dataIndex) {
                  Object.keys(node).forEach((key) => {
                      if (Object.prototype.hasOwnProperty.call(column, key)) {
                          // @ts-ignore
                          node[key] = column[key];
                      }
                      else {
                          // @ts-ignore
                          delete node[key];
                      }
                  });
              }
          });
      }
      function updateColumnByPath(path, field, value) {
          utils.setValueByPath(originColumnsRef.value, `${path}.${String(field)}`, value);
      }
      function updateColumnByDataIndex(dataIndex, field, value) {
          utils.traverseTreeBFS(originColumnsRef.value, (node) => {
              if (node.dataIndex === dataIndex) {
                  node[field] = value;
              }
          });
      }
      function changeColumnVisibleByPath(path, visible) {
          utils.setValueByPath(originColumnsRef.value, `${path}.visible`, visible);
      }
      function changeColumnVisibleByDataIndex(dataIndex, visible) {
          utils.traverseTreeBFS(originColumnsRef.value, (node) => {
              if (node.dataIndex === dataIndex) {
                  node.visible = visible;
              }
          });
      }
      return {
          ref(el) {
              tableInstance.value = el;
          },
          tableInstance,
          originColumns: originColumnsRef,
          columns: columnsRef,
          resetColumns,
          changeColumnVisibleByPath,
          changeColumnVisibleByDataIndex,
          updateColumnByPath,
          updateColumnByDataIndex,
          replaceColumnByPath,
          replaceColumnByDataIndex
      };
  }

  const defaultPagination$1 = () => {
      return {
          current: 1,
          pageSize: 10,
          total: 0,
          showTotal: true,
          showPageSize: true
      };
  };
  function useTablePage(config) {
      const loading = vue.ref(false);
      const tableData = vue.ref();
      const columnsHooks = useColumns(config.columns);
      const pagination = vue.ref(defaultPagination$1());
      function onPageChange(current) {
          pagination.value.current = current;
          return getTableData();
      }
      function onPageSizeChange(pageSize) {
          pagination.value.current = 1;
          pagination.value.pageSize = pageSize;
          return getTableData();
      }
      if (utils.isObject(config.pagination)) {
          pagination.value = Object.assign(Object.assign({}, defaultPagination$1()), config.pagination);
      }
      function getTableData() {
          loading.value = true;
          return config
              .fetch({ pageNum: pagination.value.current, pageSize: pagination.value.pageSize })
              .then((data) => {
              tableData.value = data.records;
              pagination.value.total = data.total;
          })
              .finally(() => {
              loading.value = false;
          });
      }
      /**
       * 刷新表格数据，pagination.pageNum不会变
       */
      function refreshData() {
          return getTableData();
      }
      /**
       * 加载表格数据，pagination.pageNum会设置为1
       */
      function loadTableData() {
          pagination.value.current = 1;
          return getTableData();
      }
      if (config.immediate === true) {
          loadTableData();
      }
      return {
          tableData,
          loading,
          loadTableData,
          refreshData,
          pagination,
          tableInstance: columnsHooks.tableInstance,
          originColumns: columnsHooks.originColumns,
          columns: columnsHooks.columns,
          resetColumns: columnsHooks.resetColumns,
          changeColumnVisibleByPath: columnsHooks.changeColumnVisibleByPath,
          changeColumnVisibleByDataIndex: columnsHooks.changeColumnVisibleByDataIndex,
          updateColumnByPath: columnsHooks.updateColumnByPath,
          updateColumnByDataIndex: columnsHooks.updateColumnByDataIndex,
          replaceColumnByPath: columnsHooks.replaceColumnByPath,
          replaceColumnByDataIndex: columnsHooks.replaceColumnByDataIndex,
          tableConfig: vue.reactive({
              loading,
              data: tableData,
              columns: columnsHooks.columns,
              ref: columnsHooks.ref,
              pagination,
              onPageChange,
              onPageSizeChange
          })
      };
  }

  const defaultPagination = () => {
      return {
          current: 1,
          pageSize: 10,
          total: 0,
          showTotal: true,
          showPageSize: true
      };
  };
  function useTable(config) {
      const loading = vue.ref(false);
      const tableData = vue.ref();
      const columnsHooks = useColumns(config.columns);
      const pagination = vue.ref(false);
      let onPageChange;
      let onPageSizeChange;
      if (config.pagination === true || utils.isObject(config.pagination)) {
          pagination.value = config.pagination === true ? defaultPagination() : Object.assign(Object.assign({}, defaultPagination()), config.pagination);
          onPageChange = (current) => {
              pagination.value.current = current;
          };
          onPageSizeChange = (pageSize) => {
              pagination.value.current = 1;
              pagination.value.pageSize = pageSize;
          };
      }
      /**
       * 加载表格数据
       */
      function loadTableData() {
          loading.value = true;
          return config
              .fetch()
              .then((data) => {
              tableData.value = data;
              if (config.pagination !== false) {
                  pagination.value.total = data.length;
              }
          })
              .finally(() => {
              loading.value = false;
          });
      }
      if (config.immediate === true) {
          loadTableData();
      }
      return {
          tableData,
          loading,
          loadTableData,
          pagination,
          tableInstance: columnsHooks.tableInstance,
          originColumns: columnsHooks.originColumns,
          columns: columnsHooks.columns,
          resetColumns: columnsHooks.resetColumns,
          changeColumnVisibleByPath: columnsHooks.changeColumnVisibleByPath,
          changeColumnVisibleByDataIndex: columnsHooks.changeColumnVisibleByDataIndex,
          updateColumnByPath: columnsHooks.updateColumnByPath,
          updateColumnByDataIndex: columnsHooks.updateColumnByDataIndex,
          replaceColumnByPath: columnsHooks.replaceColumnByPath,
          replaceColumnByDataIndex: columnsHooks.replaceColumnByDataIndex,
          tableConfig: vue.reactive({
              loading,
              data: tableData,
              columns: columnsHooks.columns,
              ref: columnsHooks.ref,
              pagination,
              onPageChange,
              onPageSizeChange
          })
      };
  }

  exports.useColumns = useColumns;
  exports.useMemo = useMemo;
  exports.useSelectOptions = useSelectOptions;
  exports.useTable = useTable;
  exports.useTablePage = useTablePage;

}));
