'use strict';

var vue = require('vue');
var lodashEs = require('lodash-es');
var utils = require('@devops-web/utils');

/**
 * 过滤columns，去掉visible===false的列，和children为空的列
 */
function filterColumns(columns) {
    const filteredNodes = [];
    for (const column of columns) {
        if (column.visible !== false) {
            const { children } = column;
            if (Array.isArray(children) && children.length > 0) {
                const filteredChildren = filterColumns(children);
                if (filteredChildren.length > 0) {
                    filteredNodes.push(Object.assign(Object.assign({}, column), { children: filteredChildren }));
                }
            }
            else {
                filteredNodes.push(Object.assign({}, column));
            }
        }
    }
    return filteredNodes;
}
/**
 * arco-design 扩展表格columns，增加visible，minWidth两个配置项
 * @param columns
 * @returns
 */
function useColumns(columns) {
    const originColumns = lodashEs.cloneDeep(columns);
    const originColumnsRef = vue.ref(columns);
    const tableInstance = vue.ref(null);
    let lastWidth = 0;
    const observer = new ResizeObserver((entries) => {
        const entry = entries[0];
        if (entry.contentRect.width !== lastWidth) {
            lastWidth = entry.contentRect.width;
            updateColumnsWidth();
        }
    });
    vue.watch(tableInstance, () => {
        observer.disconnect();
        if (tableInstance.value) {
            observer.observe(tableInstance.value.$el);
        }
    });
    const columnsRef = vue.ref([]);
    vue.watchEffect(() => {
        columnsRef.value = filterColumns(originColumnsRef.value);
        vue.nextTick(() => {
            updateColumnsWidth();
        });
    });
    const doFlattenColumns = (columns) => {
        const result = [];
        columns.forEach((column) => {
            if (column.children && column.children.length > 0) {
                result.push(...doFlattenColumns(column.children));
            }
            else {
                result.push(column);
            }
        });
        return result;
    };
    function getLeafColumns() {
        return doFlattenColumns(columnsRef.value);
    }
    function updateColumnsWidth() {
        if (!tableInstance.value) {
            return;
        }
        const borderWidth = tableInstance.value.bordered ? 2 : 0;
        const bodyWidth = tableInstance.value.$el.clientWidth - borderWidth;
        let bodyMinWidth = 0;
        const flattenColumns = getLeafColumns();
        const flexColumns = flattenColumns.filter((column) => typeof column.width !== 'number');
        if (flexColumns.length > 0) {
            flattenColumns.forEach((column) => {
                bodyMinWidth += Number(column.width || column.minWidth || 80);
            });
            if (bodyMinWidth <= bodyWidth) {
                const totalFlexWidth = bodyWidth - bodyMinWidth;
                if (flexColumns.length === 1) {
                    flexColumns[0].width = Number(flexColumns[0].minWidth || 80) + totalFlexWidth;
                }
                else {
                    const allColumnsWidth = flexColumns.reduce((prev, column) => prev + Number(column.minWidth || 80), 0);
                    const flexWidthPerPixel = totalFlexWidth / allColumnsWidth;
                    let noneFirstWidth = 0;
                    flexColumns.forEach((column, index) => {
                        if (index === 0)
                            return;
                        const flexWidth = Math.floor(Number(column.minWidth || 80) * flexWidthPerPixel);
                        noneFirstWidth += flexWidth;
                        column.width = Number(column.minWidth || 80) + flexWidth;
                    });
                    flexColumns[0].width = Number(flexColumns[0].minWidth || 80) + totalFlexWidth - noneFirstWidth;
                }
            }
            else {
                flexColumns.forEach((column) => {
                    column.width = Number(column.minWidth);
                });
            }
        }
        else {
            flattenColumns.forEach((column) => {
                if (!column.width && !column.minWidth) {
                    column.width = 80;
                }
                else {
                    column.width = Number(column.width || column.minWidth);
                }
            });
        }
    }
    function resetColumns() {
        originColumnsRef.value = originColumns;
    }
    function replaceColumnByPath(path, column) {
        utils.setValueByPath(originColumnsRef.value, path, column);
    }
    function replaceColumnByDataIndex(dataIndex, column) {
        utils.traverseTreeBFS(originColumnsRef.value, (node) => {
            if (node.dataIndex === dataIndex) {
                Object.keys(node).forEach((key) => {
                    if (Object.prototype.hasOwnProperty.call(column, key)) {
                        // @ts-ignore
                        node[key] = column[key];
                    }
                    else {
                        // @ts-ignore
                        delete node[key];
                    }
                });
            }
        });
    }
    function updateColumnByPath(path, field, value) {
        utils.setValueByPath(originColumnsRef.value, `${path}.${String(field)}`, value);
    }
    function updateColumnByDataIndex(dataIndex, field, value) {
        utils.traverseTreeBFS(originColumnsRef.value, (node) => {
            if (node.dataIndex === dataIndex) {
                node[field] = value;
            }
        });
    }
    function changeColumnVisibleByPath(path, visible) {
        utils.setValueByPath(originColumnsRef.value, `${path}.visible`, visible);
    }
    function changeColumnVisibleByDataIndex(dataIndex, visible) {
        utils.traverseTreeBFS(originColumnsRef.value, (node) => {
            if (node.dataIndex === dataIndex) {
                node.visible = visible;
            }
        });
    }
    return {
        ref(el) {
            tableInstance.value = el;
        },
        tableInstance,
        originColumns: originColumnsRef,
        columns: columnsRef,
        resetColumns,
        changeColumnVisibleByPath,
        changeColumnVisibleByDataIndex,
        updateColumnByPath,
        updateColumnByDataIndex,
        replaceColumnByPath,
        replaceColumnByDataIndex
    };
}

exports.useColumns = useColumns;
