import { SelectOption, SelectOptionData } from '@arco-design/web-vue';
import { Ref } from 'vue';

type FetchOptionsFunction = (...params: any[]) => Promise<SelectOption[]>;
type SelectOptionValue = string | number | boolean | Record<string, unknown>;
type FallbackFunction = (value: SelectOptionValue) => SelectOptionData;
type Config = {
    valueKey: string;
    fallback?: FallbackFunction;
    immediate: boolean;
};
/**
 * a-select 组件 options 数据的 hooks 工具
 *
 * @param fun options 数据或者一个返回 options 数据的接口函数
 * @param _config 配置
 * @param _config.valueKey 当 option 的 value 为 object 类型时，需要指定唯一标识的字段名，默认：`value`
 * @param _config.immediate 是否立即调用一次 loadOptions，默认：`false`
 * @param _config.fallback 当 value 无匹配 option 时，option 的回退函数
 *
 * @example
 * ```typescript
 * const { options } = useSelectOptions([{ label:'label-1', value:'1' }])
 *
 * // 传入函数
 * const { options, getOptions, getLabels } = useSelectOptions((params) => {
 *    console.log('传入的参数：',params)
 *    return Promise.resolve([{ label: 'label-1', value: 1 }])
 * })
 * loadOptions({type:1}) // 调用传入的函数
 *
 * // 传入函数，并立即调用loadOptions()
 * const { options, getOptions, getLabels } = useSelectOptions(() => {
 *    return Promise.resolve([{ label: 'label-1', value: 1 }])
 * },{immediate:true})
 *
 * // value为object类型
 * const { options, getOptions, getLabels } = useSelectOptions([{ label: 'label-1', value: {value:1} }])
 * getLabels(1) // label-1
 * getLabels({value:1}) // label-1
 *
 *
 * // 配置fallback，getLabels，getOptions数据来源于fallback
 * const { options, getOptions, getLabels } = useSelectOptions([], {
 *      fallback(value) {
 *        if (value === 1) {
 *          return { value, label: 'label-1' }
 *        } else {
 *          return { value, label: undefined }
 *        }
 *       }
 *    })
 * getLabels(1) // label-1
 * getOptions(1) // { value:1, label: 'label-1' }
 * getLabels(2) // undefined
 * getOptions(2) // { value:2, label: undefined }
 *
 * ```
 */
declare function useSelectOptions(fun?: FetchOptionsFunction | SelectOption[], _config?: Partial<Config>): {
    loading: Ref<boolean>;
    options: Ref<SelectOption[]>;
    /**
     * 加载options，传入immediate=true，会在初始化的时候调用一次
     */
    loadOptions: (...params: any[]) => Promise<SelectOption[]>;
    setOptions: (data: SelectOption[]) => void;
    /**
     * 根据value获取对应的option，option会从之前缓存的options选项中获取，缓存的选项没有会调用传入的fallback函数并返回fallback返回的回退option，如果还没有返回undefined
     */
    getOptions: {
        (value: undefined | SelectOptionValue): SelectOptionData | undefined;
        (value: SelectOptionValue[]): (SelectOptionData | undefined)[];
        (value: undefined | SelectOptionValue[]): (SelectOptionData | undefined)[] | undefined;
        (value: undefined | SelectOptionValue | SelectOptionValue[]): SelectOptionData | undefined | (SelectOptionData | undefined)[];
    };
    /**
     * 根据value获取对应的label，label会从之前缓存的options选项中获取，缓存的选项没有会调用传入的fallback函数并返回fallback返回的回退option.label，如果还没有返回undefined
     */
    getLabels: {
        (value: undefined | SelectOptionValue): string | undefined;
        (value: SelectOptionValue[]): (string | undefined)[];
        (value: undefined | SelectOptionValue[]): (string | undefined)[] | undefined;
        (value: undefined | SelectOptionValue | SelectOptionValue[]): string | undefined | (string | undefined)[];
    };
    fallbackOption: (value: SelectOptionValue) => SelectOptionData;
};

export { useSelectOptions };
